package de.ozgcloud.xta.client.mapper;

import java.time.ZonedDateTime;
import java.util.Optional;

import javax.xml.datatype.XMLGregorianCalendar;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.ContentType;
import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.GenericContentContainer;
import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.IsServiceAvailableValueType;
import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.LookupServiceResponse;
import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.LookupServiceResultType;
import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.MessageStatusType;
import de.ozgcloud.xta.client.gen.v3.de.xoev.transport.xta.x211.TransportReport;
import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType;
import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2008.x05.transport.MsgStatusListType;
import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2014.x10.transport.PartyIdentifierType;
import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2014.x10.transport.QualifierType;
import de.ozgcloud.xta.client.model.XtaFile;
import de.ozgcloud.xta.client.model.XtaIdentifier;
import de.ozgcloud.xta.client.model.XtaMessage;
import de.ozgcloud.xta.client.model.XtaMessageMetaData;
import de.ozgcloud.xta.client.model.XtaMessageMetaDataListing;
import de.ozgcloud.xta.client.model.XtaMessageStatus;
import de.ozgcloud.xta.client.model.XtaTransportReport;

@Mapper(
		unmappedTargetPolicy = ReportingPolicy.ERROR
)
public interface ResponseMapper {

	XtaIdentifier mapIdentifierFromPartyIdentifierType(PartyIdentifierType partyIdentifierType);

	@Mapping(target = "pendingMessageCount", source = "response.itemsPending", defaultValue = "0")
	@Mapping(target = "messages", source = "msgStatusListType.messageMetaData")
	XtaMessageMetaDataListing mapXtaMessageMetaListing(MsgStatusListType msgStatusListType, MsgBoxResponseType response);

	@Mapping(target = "service", source = "qualifier.service")
	@Mapping(target = "businessScenarioCode", source = "qualifier.businessScenario", qualifiedByName = "mapBusinessScenarioCode")
	@Mapping(target = "businessScenarioListUri", source = "qualifier.businessScenario.defined.listURI")
	@Mapping(target = "businessScenarioListVersionId", source = "qualifier.businessScenario.defined.listVersionID")
	@Mapping(target = "messageTypeCode", source = "qualifier.messageType.code")
	@Mapping(target = "messageTypePayloadSchema", source = "qualifier.messageType.payloadSchema")
	@Mapping(target = "messageId", source = "msgIdentification.messageID.value")
	@Mapping(target = "authorIdentifier", source = "originators.author.identifier")
	@Mapping(target = "readerIdentifier", source = "destinations.reader.identifier")
	@Mapping(target = "messageSize", source = "msgSize")
	XtaMessageMetaData mapXtaMessageMetaData(MessageMetaData messageMetaData);

	@Named("mapBusinessScenarioCode")
	default String mapBusinessScenarioCode(QualifierType.BusinessScenario businessScenario) {
		return businessScenario.getDefined() != null ? businessScenario.getDefined().getCode() : businessScenario.getUndefined();
	}

	default Boolean isServiceAvailable(LookupServiceResponse lookupServiceResponse) {
		return lookupServiceResponse
				.getLookupServiceResultList()
				.getLookupServiceResult()
				.stream()
				.map(LookupServiceResultType::getIsServiceAvailableValue)
				.map(IsServiceAvailableValueType::isServiceIsAvailable)
				.map(Optional::ofNullable)
				.allMatch(serviceAvailable -> serviceAvailable.orElse(false));
	}

	@Mapping(target = "content", source = "value")
	@Mapping(target = "language", source = "lang")
	@Mapping(target = "name", source = "filename")
	XtaFile mapXtaFile(ContentType contentType);

	@Mapping(target = "messageFile", source = "genericContentContainer.contentContainer.message")
	@Mapping(target = "attachmentFiles", source = "genericContentContainer.contentContainer.attachment")
	XtaMessage mapXtaMessage(GenericContentContainer genericContentContainer, MessageMetaData metaData);

	@Mapping(target = "status", source = "messageStatus")
	@Mapping(target = "metaData", source = "messageMetaData")
	XtaTransportReport mapXtaTransportReport(TransportReport transportReport);

	default ZonedDateTime mapZonedDateTime(XMLGregorianCalendar xmlGregorianCalendar) {
		return xmlGregorianCalendar.toGregorianCalendar().toZonedDateTime();
	}

	default XtaMessageStatus mapXtaMessageStatus(MessageStatusType statusType) {
		return switch (statusType.getStatus().intValue()) {
			case 0 -> XtaMessageStatus.OPEN;
			case 1 -> XtaMessageStatus.GREEN;
			case 2 -> XtaMessageStatus.YELLOW;
			case 3 -> XtaMessageStatus.RED;
			default -> null;
		};
	}
}
