package de.ozgcloud.xta.client.factory;

import static de.ozgcloud.xta.client.mapper.RequestMapper.*;

import java.math.BigInteger;

import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import de.ozgcloud.xta.client.gen.v3.eu.osci.ws.x2014.x10.transport.PartyIdentifierType;
import de.ozgcloud.xta.client.model.XtaIdentifier;

public class MessageMetaDataTestFactory {

	public static final String MESSAGE_SERVICE = "message-service";

	public static final String MESSAGE_TYPE_PAYLOAD_SCHEMA = "message-type-payload-schema";
	public static final String BUSINESS_SCENARIO_CODE = "businessScenarioCode";
	public static final String UNDEFINED_BUSINESS_SCENARIO_CODE = "undefinedBusinessScenarioCode";
	public static final String BUSINESS_SCENARIO_LIST_URI = "urn:de:dataport:codeliste:business.scenario";
	public static final String BUSINESS_SCENARIO_LIST_VERSION_ID = "1.0";

	public static final XtaIdentifier AUTHOR_IDENTIFIER = XtaIdentifier.builder()
			.value("authorIdentifierPrefix:authorIdentifierSuffix")
			.name("authorIdentifierName")
			.category("authorIdentifierCategory")
			.build();
	public static final XtaIdentifier READER_IDENTIFIER = XtaIdentifier.builder()
			.value("readerIdentifierPrefix:readerIdentifierSuffix")
			.name("readerIdentifierName")
			.category("readerIdentifierCategory")
			.build();

	public static final String MESSAGE_ID = "messageId1";
	public static final String MESSAGE_ID2 = "messageId2";
	public static final String MESSAGE_ID3 = "messageId3";

	public static final String MESSAGE_TYPE_CODE = "messageType";
	public static final String MESSAGE_TYPE_CODE2 = "messageType2";
	public static final String MESSAGE_TYPE_CODE3 = "messageType3";

	public static final BigInteger MESSAGE_SIZE = BigInteger.valueOf(1234);

	public static MessageMetaData create() {
		return createWithDefined(true);
	}

	public static MessageMetaData createWithDefined(boolean businessScenarioDefined) {
		return create(MESSAGE_ID, MESSAGE_TYPE_CODE, MESSAGE_SIZE, businessScenarioDefined);

	}

	public static MessageMetaData create(String messageId, String messageType, BigInteger messageSize, boolean businessScenarioDefined) {
		var messageMetaData = new MessageMetaData();

		{
			var messageIdType = messageMetaData.getMsgIdentification().getMessageID();
			messageIdType.setValue(messageId);
		}
		{
			var qualifier = messageMetaData.getQualifier();
			qualifier.setService(MESSAGE_SERVICE);

			{
				var messageTypeObj = qualifier.getMessageType();
				messageTypeObj.setPayloadSchema(MESSAGE_TYPE_PAYLOAD_SCHEMA);
				messageTypeObj.setListURI(MESSAGE_TYPE_LIST_URI);
				messageTypeObj.setListVersionID(MESSAGE_TYPE_LIST_VERSION_ID);
				messageTypeObj.setCode(messageType);
			}

			if (businessScenarioDefined) {
				var definedBusinessScenario = qualifier.getBusinessScenario().getDefined();
				definedBusinessScenario.setCode(BUSINESS_SCENARIO_CODE);
				definedBusinessScenario.setListURI(BUSINESS_SCENARIO_LIST_URI);
				definedBusinessScenario.setListVersionID(BUSINESS_SCENARIO_LIST_VERSION_ID);
			} else {
				qualifier.getBusinessScenario().setDefined(null);
				qualifier.getBusinessScenario().setUndefined(UNDEFINED_BUSINESS_SCENARIO_CODE);
			}
		}

		assignIdentifier(AUTHOR_IDENTIFIER, messageMetaData.getOriginators().getAuthor().getIdentifier());
		assignIdentifier(READER_IDENTIFIER, messageMetaData.getDestinations().getReader().getIdentifier());

		messageMetaData.setMsgSize(messageSize);

		return messageMetaData;
	}

	private static void assignIdentifier(XtaIdentifier identifier, PartyIdentifierType target) {
		target.setType(IDENTIFIER_TYPE);
		target.setValue(identifier.value());
		target.setName(identifier.name());
		target.setCategory(identifier.category());
	}
}
